/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;
import org.kemet.tests.basic.TestDAL;
import org.kemet.tests.basic.TestGrammarManager;
import org.kemet.tests.basic.TestNotUsedClassInstances;
import org.kemet.tests.basic.TestOthers;
import org.kemet.tests.basic.TestReflexion;
import org.kemet.tests.composite.TestPartialSequence;
import org.kemet.tests.determiner.TestDeterminer;
import org.kemet.tests.integration.transliteration.TestsToHieroglyphs;
import org.kemet.tests.integration.transliteration.TestsToTransliteration;
import org.kemet.tests.integration.transliteration.TestsToType;
import org.kemet.tests.location.TestMdC;
import org.kemet.tests.phonetic.TestsToPhonetic;
import org.kemet.tests.phonograms.TestBiliteral;
import org.kemet.tests.phonograms.TestTriliteral;
import org.kemet.tests.phonograms.TestUniliteral;
import org.kemet.tests.ppa.TestsPPA;
import org.kemet.tests.xml.HieroTests;
import org.kemet.tests.xml.PhonogramsTests;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>Tests suite.</b>
 * </p>
 * 
 * @since 0.1
 */
public class AllTests {
	static Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * 
	 * <p>
	 * <b>execute the tests suite.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	public static Test suite() {
		log.info("system  = " + System.getProperty("os.name"));
		log.info("TestSuite start...");

		TestSuite suite = new TestSuite("Tests for org.kemet.tests");

		// execute the basic tests...
		suite.addTestSuite(TestDAL.class);
		log.info("		[TestDAL] 						exec");

		suite.addTestSuite(TestReflexion.class);
		log.info("		[TestReflexion] 				exec");

		suite.addTestSuite(TestNotUsedClassInstances.class);
		log.info("		[TestNotUsedClassInstances]		exec");

		suite.addTestSuite(TestGrammarManager.class);
		log.info("		[TestGrammarManager] 			exec");

		suite.addTestSuite(TestOthers.class);
		log.info("		[TestOthers]					exec");

		// execute the tests for uniliteral, biliteral and triliteral...
		suite.addTestSuite(TestUniliteral.class);
		log.info("		[TestUniliteral]				exec");

		suite.addTestSuite(TestBiliteral.class);
		log.info("		[TestBiliteral]					exec");

		suite.addTestSuite(TestTriliteral.class);
		log.info("		[TestTriliteral]				exec");

		// execute the tests for determiners...
		suite.addTestSuite(TestDeterminer.class);
		log.info("		[TestDeterminer]				exec");

		// execute the tests for PPA...
		suite.addTestSuite(TestsPPA.class);
		log.info("		[TestsPPA]						exec");

		// execute the tests for location manager...
		suite.addTestSuite(TestMdC.class);
		log.info("		[TestMdC]						exec");

		// execute the integration tests...
		suite.addTestSuite(TestsToHieroglyphs.class);
		log.info("		[TR- TestsToHieroglyphs]		exec");

		suite.addTestSuite(TestsToTransliteration.class);
		log.info("		[TR - TestsToTransliteration] 	exec");

		suite.addTestSuite(TestsToType.class);
		log.info("		[TR - TestsToType]				exec");

		suite
				.addTestSuite(org.kemet.tests.integration.hieroglyph.TestsToTransliteration.class);
		log.info("		[HI - TestsToTransliteration] 	exec");

		suite
				.addTestSuite(org.kemet.tests.integration.hieroglyph.TestsToType.class);
		log.info("		[HI - TestsToType] 				exec");

		suite.addTestSuite(TestsToPhonetic.class);
		log.info("		[HI - TestsToPhonetic]			exec");

		suite.addTestSuite(TestPartialSequence.class);
		log.info("		[TestPartialSequence]			exec");

		// execute the hieroglyphs tests...
		suite.addTestSuite(HieroTests.class);
		log.info("		[HieroTests]					exec");

		suite.addTestSuite(PhonogramsTests.class);
		log.info("		[PhonogramsTests]				exec");

		log.info("TestSuite end");

		return suite;
	}
}
