/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.phonograms;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.kemet.tests.TestRoot;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>fonctionnal tests for triliteral.</b>
 * </p>
 * 
 * @since 0.1
 * 
 */
public class TestTriliteral extends TestRoot {
	static final Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>sequence for test.</b>
	 * </p>
	 */
	private String[] d1;

	/**
	 * <p>
	 * <b>expected result.</b>
	 * </p>
	 */
	private String[] d1Expected;

	/**
	 * <p>
	 * <b>set the parameters before execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	public void init() {
		// initialize the data tests...
		d1 = new String[] { "jwnn", "jm33", "jswsw", "cnh)h)", "wwh.m", "md_h." };

		d1Expected = new String[] { "jwn", "jm3", "jsw", "cnh)", "wh.m",
				"md_h." };
	}

	/**
	 * <p>
	 * <b>test : detect and epurate phonetic complements.</b>
	 * </p>
	 * 
	 * throws Exception
	 * 
	 * @since 0.1
	 */
	@Test
	public void testTriEx1() throws Exception {
		assertEquals(Arrays.equals(gm.toEpuratePhoneticComplementFromSequence(
				d1, 3, phInfo, args), d1Expected), true);
	}

	/**
	 * <p>
	 * <b>test : phonetic read.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testTriEx2() {
		// tests to pronunciation cases
		try {
			assertEquals(gm.toPhoneticReadWithGlobalApproach(new String[] {
					"jwn", "jm3" }, false), "");
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>test : transliteration to hieroglyphs or hieroglyphs to
	 * transliteration.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testTriEx3() {
		// tests triliteres detection in sequence (signList or transliterated
		// sequence)
		gm.setNumberOfConsomns(TRILITERAL);

		try {
			assertEquals(gm.toHieroglyphs(new String[] { "jwn", "md_h." }),
					"O28S10");
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			assertEquals(
					gm.toTransliteration(new String[] { "O28", "S10" }, 0),
					"jwnmd_h.");
		} catch (Exception e) {
			assertTrue(false);
		}
	}
}
