/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.xml;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.kemet.tests.IAttr;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>hieroglyphs Tests : test if there are double id and if the hieroglyph
 * files exist.</b>
 * </p>
 * 
 * @since 0.1
 * 
 */
public class HieroTests extends TestCase implements IAttr {
	static Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>current XPath.</b>
	 * </p>
	 */
	private XPath xpath;

	/**
	 * <p>
	 * <b>current DOM document.</b>
	 * </p>
	 */
	private Document document;

	/**
	 * <p>
	 * <b>current DOM document 2.</b>
	 * </p>
	 */
	private Document document2;

	/**
	 * <p>
	 * <b>current xpath expression.</b>
	 * </p>
	 */
	private String expression;

	/**
	 * <p>
	 * <b>set the parameters before execute the tests.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.1
	 */
	@Before
	public void setUp() throws Exception {
		// parse the XML as a W3C Document
		DocumentBuilder builder = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();

		document = builder.parse(new File(XMLPath + "\\fr\\hiero_fr.xml"));

		document2 = builder.parse(new File(XMLPath + "\\en\\hiero_en.xml"));

		xpath = XPathFactory.newInstance().newXPath();
	}

	/**
	 * <p>
	 * <b>test : compare the number of hieroglyphs between FR and EN files.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test1CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//id)", new InputSource(new FileReader(XMLPath
						+ "\\fr\\hiero_fr.xml")), XPathConstants.NUMBER);

		log.info("number hiero = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//id)", new InputSource(new FileReader(XMLPath
						+ "\\en\\hiero_en.xml")), XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number <hieroglyph>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test2CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//hieroglyph)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		Double elementCount2 = (Double) xPath
				.evaluate("count(//hieroglyph)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <Transliteration>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test3CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//Transliteration)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number Transliteration = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//Transliteration)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <value>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test4CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//value)", new InputSource(new FileReader(
						XMLPath + "\\fr\\hiero_fr.xml")), XPathConstants.NUMBER);

		log.info("number value = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//value)", new InputSource(new FileReader(
						XMLPath + "\\en\\hiero_en.xml")), XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of determiner.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test5CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//Sign[@determiner])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number determiner = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//Sign[@determiner])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of ideogram.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test6CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//Sign[@ideogram])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number ideogram = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//Sign[@ideogram])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of substitute.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test7CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//Sign[@substitute])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number substitute = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//Sign[@substitute])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of usedForIdeogram.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test8CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath.evaluate(
				"count(//Sign[@usedForIdeogram])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
				XPathConstants.NUMBER);

		log.info("number usedForIdeogram = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath.evaluate(
				"count(//Sign[@usedForIdeogram])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
				XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of feminineGrammaticalKind.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test9CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath.evaluate(
				"count(//Sign[@feminineGrammaticalKind])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
				XPathConstants.NUMBER);

		log
				.info("number feminineGrammaticalKind = "
						+ elementCount1.intValue());

		Double elementCount2 = (Double) xPath.evaluate(
				"count(//Sign[@feminineGrammaticalKind])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
				XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of indicator_ppa.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test10CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath.evaluate(
				"count(//Sign[@indicator_ppa])", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
				XPathConstants.NUMBER);

		log.info("number indicator_ppa = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath.evaluate(
				"count(//Sign[@indicator_ppa])", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
				XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <identification>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test11CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//identification)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number identification = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//identification)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <suffixPronoun>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test12CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//suffixPronoun)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number suffixPronoun = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//suffixPronoun)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <demonstrativePronoun>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test13CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath.evaluate(
				"count(//demonstrativePronoun)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
				XPathConstants.NUMBER);

		log.info("number demonstrativePronoun = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath.evaluate(
				"count(//demonstrativePronoun)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
				XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : compare the number of <preposition>.</b>
	 * </p>
	 * 
	 * @throws Exception
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test14CompareFiles() throws Exception {
		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPath xPath = xPathFactory.newXPath();

		Double elementCount1 = (Double) xPath
				.evaluate("count(//preposition)", new InputSource(
						new FileReader(XMLPath + "\\fr\\hiero_fr.xml")),
						XPathConstants.NUMBER);

		log.info("number preposition = " + elementCount1.intValue());

		Double elementCount2 = (Double) xPath
				.evaluate("count(//preposition)", new InputSource(
						new FileReader(XMLPath + "\\en\\hiero_en.xml")),
						XPathConstants.NUMBER);

		assertEquals(elementCount1.intValue(), elementCount2.intValue());
	}

	/**
	 * <p>
	 * <b>test : verify if there are double id.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testDoubleID() {
		NodeList node;
		Node id;

		try {
			expression = "/HieroCollection/Signs/Sign";
			node = (NodeList) xpath.evaluate(expression, document,
					XPathConstants.NODESET);

			XPath xpath = XPathFactory.newInstance().newXPath();
			expression = "id";

			// insert ID to arrayList...
			ArrayList sortList = new ArrayList();

			for (int i = 0; i < node.getLength(); i++) {
				id = (Node) xpath.evaluate(expression, node.item(i),
						XPathConstants.NODE);
				sortList.add(id.getFirstChild().getNodeValue());
			}

			Collections.sort(sortList);

			// compute the "double" id
			ArrayList doub = new ArrayList();

			String previousValue = "";
			for (int i = 0; i < sortList.size(); i++) {
				if (!sortList.get(i).equals(previousValue))
					previousValue = sortList.get(i).toString();
				else
					doub.add(sortList.get(i));
			}

			// display the collection...
			if (doub.size() > 0) {
				log.info("objets count : " + doub.size());
				for (int i = 0; i < doub.size(); i++) {
					log.info(doub.get(i));
				}

				assertEquals(doub.size(), 0);
			}

		} catch (XPathExpressionException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>test : verify if the hieroglyph file exists.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testExistFile() {
		NodeList node;
		Node id;

		try {
			expression = "/HieroCollection/Signs/Sign";
			node = (NodeList) xpath.evaluate(expression, document,
					XPathConstants.NODESET);

			XPath xpath = XPathFactory.newInstance().newXPath();
			expression = "hieroglyph";

			// insert hieroglyph to arrayList...
			ArrayList sortList = new ArrayList();

			for (int i = 0; i < node.getLength(); i++) {
				id = (Node) xpath.evaluate(expression, node.item(i),
						XPathConstants.NODE);
				sortList.add(id.getFirstChild().getNodeValue());
			}

			Collections.sort(sortList);

			// verify if file exist ?
			ArrayList notExist = new ArrayList();
			String value = "";

			for (int i = 0; i < sortList.size(); i++) {
				value = sortList.get(i).toString();

				if (!value.equals("---")) {
					File file = new File(XMLPath + value);

					if (!file.exists())
						notExist.add(sortList.get(i));
				}
			}

			// display the collection...
			if (notExist.size() > 0) {
				log.info("objets count : " + notExist.size());
				for (int i = 0; i < notExist.size(); i++) {
					log.info(notExist.get(i));
				}
				assertEquals(notExist.size(), 0);
			}

		} catch (XPathExpressionException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>test : verify if there are double id.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testDoubleID_2() {
		NodeList node;
		Node id;

		try {
			expression = "/HieroCollection/Signs/Sign";
			node = (NodeList) xpath.evaluate(expression, document2,
					XPathConstants.NODESET);

			XPath xpath = XPathFactory.newInstance().newXPath();
			expression = "id";

			// insert ID to arrayList...
			ArrayList sortList = new ArrayList();

			for (int i = 0; i < node.getLength(); i++) {
				id = (Node) xpath.evaluate(expression, node.item(i),
						XPathConstants.NODE);
				sortList.add(id.getFirstChild().getNodeValue());
			}

			Collections.sort(sortList);

			// compute the "double" id
			ArrayList doub = new ArrayList();

			String previousValue = "";
			for (int i = 0; i < sortList.size(); i++) {
				if (!sortList.get(i).equals(previousValue))
					previousValue = sortList.get(i).toString();
				else
					doub.add(sortList.get(i));
			}

			// display the collection...
			if (doub.size() > 0) {
				log.info("objets count : " + doub.size());
				for (int i = 0; i < doub.size(); i++) {
					log.info(doub.get(i));
				}

				assertEquals(doub.size(), 0);
			}

		} catch (XPathExpressionException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>test : verify if the hieroglyph file exists.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testExistFile_2() {
		NodeList node;
		Node id;

		try {
			expression = "/HieroCollection/Signs/Sign";
			node = (NodeList) xpath.evaluate(expression, document2,
					XPathConstants.NODESET);

			XPath xpath = XPathFactory.newInstance().newXPath();
			expression = "hieroglyph";

			// insert hieroglyph to arrayList...
			ArrayList sortList = new ArrayList();

			for (int i = 0; i < node.getLength(); i++) {
				id = (Node) xpath.evaluate(expression, node.item(i),
						XPathConstants.NODE);
				sortList.add(id.getFirstChild().getNodeValue());
			}

			Collections.sort(sortList);

			// verify if file exist ?
			ArrayList notExist = new ArrayList();
			String value = "";

			for (int i = 0; i < sortList.size(); i++) {
				value = sortList.get(i).toString();

				if (!value.equals("---")) {
					File file = new File(XMLPath + value);

					if (!file.exists())
						notExist.add(sortList.get(i));
				}
			}

			// display the collection...
			if (notExist.size() > 0) {
				log.info("objets count : " + notExist.size());
				for (int i = 0; i < notExist.size(); i++) {
					log.info(notExist.get(i));
				}
				assertEquals(notExist.size(), 0);
			}

		} catch (XPathExpressionException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>test : verify if the hieroglyph file exists in XML file.</b>
	 * </p>
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void testNumberOfHierosNotInserted() {
		NodeList node;
		int nbFr = 0, nbEn = 0;

		ArrayList hr = new ArrayList();
		ArrayList hr2 = new ArrayList();

		// read all hieroglyph files...
		File rep = new File(XMLPath + "\\hieroglyphs\\");

		if (rep.exists()) {
			String list[];
			list = rep.list();

			for (int i = 0; i < list.length; i++) {
				File hiero = new File(list[i]);

				if (hiero.getName().contains("hiero_")) {
					try {
						// verify if exist in XML file (fr and en)...
						expression = "/HieroCollection/Signs/Sign[hieroglyph='hieroglyphs/"
								+ hiero.getName() + "']";
						node = (NodeList) xpath.evaluate(expression, document,
								XPathConstants.NODESET);

						if (node.getLength() == 0) {
							hr.add(hiero.getName());
							nbFr++;
						}

						node = (NodeList) xpath.evaluate(expression, document2,
								XPathConstants.NODESET);

						if (node.getLength() == 0) {
							hr2.add(hiero.getName());
							nbEn++;
						}
					} catch (XPathExpressionException e) {
						assertTrue(false);
					}
				}
			}

			// generate XML parts...
			if (nbFr > 0) {
				log.info("number of hieroglyphs (fr) not exist = " + nbFr);

				FileWriter fw;
				try {
					fw = new FileWriter(
							"C:\\Documents and Settings\\Fabrice.FABRICEX-EB682E\\Bureau\\xmlFr.txt",
							true);
					BufferedWriter output = new BufferedWriter(fw);

					for (int i = 0; i < hr.size(); i++) {
						output.write("<Sign>");

						int pos = ((String) hr.get(i)).indexOf(".");
						String id = ((String) hr.get(i)).substring(6, pos);

						output.write("<id>" + id + "</id>");

						output.write("<hieroglyph>hieroglyphs/" + hr.get(i)
								+ "</hieroglyph>");

						output.write("</Sign>");
					}

					output.flush();
					output.close();
				} catch (IOException e) {
					assertTrue(false);
				}

				assertTrue(false);
			}

			if (nbEn > 0) {
				log.info("number of hieroglyphs (en) not exist = " + nbEn);

				FileWriter fw;
				try {
					fw = new FileWriter(
							"C:\\Documents and Settings\\Fabrice.FABRICEX-EB682E\\Bureau\\xmlEn.txt",
							true);
					BufferedWriter output = new BufferedWriter(fw);

					for (int i = 0; i < hr2.size(); i++) {
						output.write("<Sign>");

						int pos = ((String) hr2.get(i)).indexOf(".");
						String id = ((String) hr2.get(i)).substring(6, pos);

						output.write("<id>" + id + "</id>");

						output.write("<hieroglyph>hieroglyphs/" + hr2.get(i)
								+ "</hieroglyph>");

						output.write("</Sign>");
					}

					output.flush();
					output.close();
				} catch (IOException e) {
					assertTrue(false);
				}

				assertTrue(false);
			}
		}
	}
}
